/*
QUSOFT MICROSYSTMES
1869

Classe: TShip - Type Ship

Hirarchie:
TObject -> TShip
*/

/*
		Copyright 2001 Frdric Brown
*/

/*
    This file is part of Gundealer.

    Gundealer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gundealer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gundealer; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * The TShip class represents a 1869's ship.
*/

#define Ship_TYPE_SCHOONER 0
#define Ship_TYPE_BARK 1
#define Ship_TYPE_STEAMSHIP 2
#define Ship_TYPE_LARGE_STEAMSHIP 3
#define Ship_TYPE_BRIG 4
#define Ship_TYPE_FRIGATE 5
#define Ship_TYPE_CLIPPER 6
#define Ship_TYPE_STEEL_CLIPPER 7

#define Ship_HULL_WOOD 0
#define Ship_HULL_IRON 1

//Structure de l'objet de Type Ship extension de TObject
typedef struct Ship {
	char *class; //Le nom de la classe
	char *super; //Le nom de la superclasse
	struct Ship* 		(*clone) 		(struct Ship*);
	BOOL 						(*equals) 	(struct Object*, struct Object*);
	void 						(*finalize) (struct Ship*);
	TString* 				(*toString) (struct Object*);
	TString* name;
	TPort* port;
	float money;
	TVector* cargo;
	int age;
	int capacity;
	int hull;
	int maxPass;
	int maxCrew;
	BOOL crewed;
	int experience;
	int delivery;
	float price;
	int type;
} TShip;

//Dclaration des constructeurs
TShip* TShip0(int, int, int, int, int, float, int);

//Dclaration des mthodes
void		 	TShip_finalize	(TShip*);
TShip*	 	TShip_clone			(TShip*);

/**
 * Ce constructeur instancie un objet de Type Ship.
 @since Release 1.0
*/
TShip* TShip0 (int capacity, int hull, int maxPass, int maxCrew, int delivery, float price, int type) {
	TShip *this = (TShip*)malloc(sizeof(TShip));
	
	this->class  = "TShip";
	this->super  = "TObject";
	
	this->clone    = TShip_clone;
	this->equals   = TObject_equals;
	this->finalize = TShip_finalize;
	this->toString = TObject_toString;
	
	this->name 	= s("");
	this->port 	= NULL;
	this->money	= 0;
	this->cargo	= TVector0();
	this->age = 0;
	
	this->capacity = capacity;
	this->hull = hull;
	this->maxPass = maxPass;
	this->maxCrew = maxCrew;
	this->crewed = FALSE;
	this->experience = 0;
	this->delivery = delivery;
	this->price = price;
	this->type = type;
	
	return this;
}

void TShip_finalize	(TShip* this)
{
	
	int i;
	TObject *obj;
	this->name->finalize(this->name);
	
	for (i = 0; i < this->cargo->size; i++)
	{
		obj = (TObject*)this->cargo->elementAt(this->cargo, i);
		obj->finalize(obj);
	}
	
	this->cargo->finalize(this->cargo);
	
	free(this);
}

TShip* TShip_clone (TShip* this)
{
	TShip* clone;
	
	clone = TShip0(this->capacity, this->hull, this->maxPass, this->maxCrew, this->delivery, this->price, this->type);
	
	return clone;
}
