/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TCursor - Type Cursor

Hirarchie:
TObject -> TComponent -> TCursor
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TCursor permet  l'utilisateur d'utiliser un curseur pour interagir
 * avec l'environnement graphique.
 @since Release 1.0
*/

#define Cursor_DEFAULT_CURSOR Cursor_ARROW_CURSOR
#define Cursor_DEFAULT_CURSOR_MASK Cursor_ARROW_CURSOR_MASK
#define Cursor_DEFAULT_SPEED 4

//Curseur en flche
const unsigned char Cursor_ARROW_CURSOR[] = {0xFF, 0xFE, 0xFC, 0xF8, 0xFC, 0xEE, 0xC7, 0x83};

//Masque du curseur en flche
const unsigned char Cursor_ARROW_CURSOR_MASK[] = {0x00, 0x7D, 0x7B, 0x77, 0x6B, 0x55, 0x3A, 0x7C};

//Curseur en flche qui pointe vers le nord
const unsigned char Cursor_UP_ARROW_CURSOR[] = {0x18, 0x3C, 0x7E, 0xFF, 0x18, 0x18, 0x18, 0x18};

//Curseur en flche qui pointe vers le sud
const unsigned char Cursor_DOWN_ARROW_CURSOR[] = {0x18, 0x18, 0x18, 0x18, 0xFF, 0x7E, 0x3C, 0x18};

//Curseur en point
const unsigned char Cursor_POINT_CURSOR[] = {0x00, 0x18, 0x3C, 0x7E, 0x7E, 0x3C, 0x18, 0x00};

//Curseur en sablier
const unsigned char Cursor_HOURGLASS_CURSOR[] = {0xFF, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0xFF};

//Curseur en croix
const unsigned char Cursor_CROSS_CURSOR[] = {0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81};

//Curseur en plus
const unsigned char Cursor_PLUS_CURSOR[] = {0x18, 0x18, 0x18, 0xFF, 0xFF, 0x18, 0x18, 0x18};

//Structure de l'objet de Type Cursor extension de TComponent
typedef struct Cursor {
	char* class; 	//Le nom de la classe
	char* super; 	//Le nom de la superclasse
	struct Cursor*		(*clone)		(struct Cursor*);
	BOOL							(*equals)		(struct Component*, struct Component*);
	void							(*finalize)	(struct Cursor*);
	TString* 					(*toString) (struct Component*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Cursor*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	unsigned char* cursor; 	//L'image du curseur
	unsigned char* mask; 	//Le masque de l'image du curseur
	int speedX;							//La vitesse horizontale
	int speedY;							//La vitesse verticale
	BOOL xor;								//Spcifie si le curseur est en mode XOR
	BOOL mnemonic;				//Spcifie si il y a prsence de mnmoniques
	void							(*use)				(struct Cursor*);
	void							(*setCursor0)	(struct Cursor*, unsigned char*, unsigned char*);
	void							(*setCursor1)	(struct Cursor*, unsigned char*);
	void							(*setXor)			(struct Cursor*, BOOL);
	BOOL							(*isOver)			(struct Cursor*, TComponent*);
} TCursor;

//Dclaration des constructeurs
TCursor* TCursor0();

//Dclaration des mthodes
TCursor*		TCursor_clone				(TCursor*);
void				TCursor_finalize		(TCursor*);
void				TCursor_print				(TCursor*);
int					TCursor_use					(TCursor*);
void				TCursor_setCursor0	(TCursor*, unsigned char*, unsigned char*);
void				TCursor_setCursor1	(TCursor*, unsigned char*);
void				TCursor_setXor			(TCursor*, BOOL);
BOOL				TCursor_isOver			(TCursor*, TComponent*);

/**
 * Ce constructeur instancie un objet de Type Cursor.
 @since Release 1.0
*/
TCursor* TCursor0 () {
	TCursor *this = (TCursor*)malloc(sizeof(TCursor));
	
	this->class  = "TCursor";
	this->super  = "TComponent";
	
	this->height 		= 8;
	this->visible 	= FALSE;
	this->width 		= 8;
	this->x					= 0;
	this->y					= 0;
	this->img				= (BITMAP*)malloc(sizeof(BITMAP));
	this->cursor  	= Cursor_DEFAULT_CURSOR;
	this->mask	  	= Cursor_DEFAULT_CURSOR_MASK;
	this->speedX		= Cursor_DEFAULT_SPEED;
	this->speedY		= Cursor_DEFAULT_SPEED;
	this->xor				= FALSE;
	this->mnemonic 	= FALSE;
	
	this->clone 	 = TCursor_clone;
	this->equals 	 = TComponent_equals;
	this->finalize = TCursor_finalize;
	this->toString = TComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TCursor_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;

	this->use 			= TCursor_use;
	this->setCursor0 = TCursor_setCursor0;
	this->setCursor1 = TCursor_setCursor1;
	this->setXor		= TCursor_setXor;
	this->isOver		= TCursor_isOver;

	return this;
}

/**
 * Cette mthode retourne un objet de Type Cursor qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TCursor* TCursor_clone(TCursor* this) {

	TCursor *clone;
	
	clone = TCursor0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->cursor		= this->cursor;
	clone->mask			= this->mask;
	clone->speedX		= this->speedX;
	clone->speedY		= this->speedY;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TCursor_finalize (TCursor* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		free(this->img);
		free(this);	
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TCursor_print(TCursor* this) {
	
	int attr;
	
	if (this->xor) {
		attr = SPRT_XOR;
	}
	else {
		attr = SPRT_OR;
	}
	
	Sprite8(this->x, this->y, 8, this->cursor, LCD_MEM, attr);
	
	if (this->mask != NULL) {
		Sprite8(this->x, this->y, 8, this->mask, LCD_MEM, SPRT_AND);
	}
}

/**
 * Cette mthode permet  l'utilisateur d'utiliser le composant.
 @since Release 1.0
*/
int TCursor_use(TCursor* this) {
	
	//void *kbq = kbd_queue ();
	int x0 = this->x;
	int y0 = this->y;
	unsigned short key = 0;
	int pasX = this->speedX;
	int pasY = this->speedY;
	unsigned short key_right = KEY_RIGHT;
	unsigned short key_left = KEY_LEFT;
	unsigned short key_up = KEY_UP;
	unsigned short key_down = KEY_DOWN;
	unsigned short key_up_right = key_up | key_right;
	unsigned short key_up_left = key_up | key_left;
	unsigned short key_down_right = key_down | key_right;
	unsigned short key_down_left = key_down | key_left;
	unsigned short key_enter = KEY_ENTER;
	unsigned short lcd_width = LCD_WIDTH;
	unsigned short lcd_height = LCD_HEIGHT;
	
	this->setVisible(this, TRUE);
	
	do {
		if (kbhit()) { //if (!OSdequeue (&key, kbq))
			key = ngetchx();
			
			//GKeyFlush();
			
			if (key == key_right) {
				x0 = x0 + pasX;
			}
			else if (key == key_left) {
				x0 = x0 - pasX;
			}
			else if (key == key_up) {
				y0 = y0 - pasY;
			}
			else if (key == key_down) {
				y0 = y0 + pasY;
			}
			else if (key == key_down_right) {
				y0 = y0 + pasY;
				x0 = x0 + pasX;
			}
			else if (key == key_down_left) {
				y0 = y0 + pasY;
				x0 = x0 - pasX;
			}
			else if (key == key_up_right) {
				y0 = y0 - pasY;
				x0 = x0 + pasX;
			}
			else if (key == key_up_left) {
				y0 = y0 - pasY;
				x0 = x0 - pasX;
			}
			else if (this->mnemonic) {
				break;
			}
			
			if (x0 > lcd_width) {
				x0 = lcd_width;
			}
			
			if (y0 > lcd_height) {
				y0 = lcd_height;
			}
			
			if (x0 < 0) {
				x0 = 0;
			}
			
			if (y0 < 0) {
				y0 = 0;
			}
			
			this->move(this, x0, y0);
		}
	} while(key != key_enter);
	
	this->setVisible(this, FALSE);
	
	return key;
}

/**
 * Cette mthode change l'image du cursor.
 @param cursor L'image du curseur
 @param mask L'image du masque du curseur (NULL si N/A)
 @since Release 1.0
*/
void TCursor_setCursor0(TCursor* this, unsigned char* cursor, unsigned char* mask) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->cursor = cursor;
	this->mask	 = mask;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change l'image du cursor.
 @param cursor L'image du curseur
 @since Release 1.0
*/
void TCursor_setCursor1(TCursor* this, unsigned char* cursor) {

	this->setCursor0(this, cursor, NULL);
}

/**
 * Cette mthode permet de savoir si le curseur est par dessus un composant
 * graphique particulier (cible).
 @param component0 La cible
 @return result - Vrai si le composant est par dessus
 @since Release 1.0
*/
BOOL TCursor_isOver (TCursor* this, TComponent* component0) {
	
	BOOL result = FALSE;
	
	if (this->x >= component0->x &&
			this->x < component0->x + component0->width &&
			this->y >= component0->y &&
			this->y < component0->y + component0->height
		 ) {
		 	result = TRUE;
	}
	
	return result;
}

/**
 * Cette mthode permet de spcifier si le curseur est en mode XOR.
 @param xor Spcifie si le curseur est en mode XOR.
 @since Release 1.0
*/
void TCursor_setXor (TCursor* this, BOOL xor) {
	
	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->xor = xor;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
