/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TFile - Type File

Hirarchie:
TObject -> TFile
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TFile implmente des mthodes permettant l'criture
 * formate des fichiers binaires.
 @since Release 1.0
*/

//Mode criture seulement
#define File_WRITE 0

//Mode cration pour mise  jour (lecture-criture)
#define File_UPDATE 1

//Mode extension
#define File_EXPEND 2

//Mode extension avec mise  jour
#define File_EXPENDUPDATE 3

//Mode criture seulement
#define File_READ 4

//Mode ouverture pour mise  jour
#define File_READUPDATE 5

//Structure de l'objet de Type File extension de TObject
typedef struct File {
	char *class;  //Le nom de la classe
	char *super;  //Le nom de la superclasse
	struct Object* 	(*clone) 		(struct Object*);
	BOOL 						(*equals) 	(struct Object*, struct Object*);
	void  					(*finalize)	(struct File*);
	TString* 				(*toString) (struct Object*);
	FILE *file;   //Le fichier gr
	TString *name;//Le nom du fichier
	BOOL opened;  //Indique si le fichier est ouvert
	int mode;		  //Spcifie le mode d'criture
	void  		(*close)				(struct File*);
	//long			(*flush 			(struct File*);
	void  		(*open)					(struct File*);
	BOOL			(*exists)				(struct File*);
	BOOL			(*readBoolean) 	(struct File*);
	char			(*readChar)			(struct File*);
	char*			(*readChars) 		(struct File*, long);
	double		(*readDouble) 	(struct File*);
	float			(*readFloat) 		(struct File*);
	int				(*readInt) 			(struct File*);
	long			(*readLong) 		(struct File*);
	TString* 	(*readString)		(struct File*);
	//long			(*size 				(struct File*);
	void			(*writeBoolean) (struct File*, BOOL);
	void			(*writeChar)		(struct File*, char);
	void			(*writeChars) 	(struct File*, char*);
	void			(*writeDouble) 	(struct File*, double);
	void			(*writeFloat) 	(struct File*, float);
	void			(*writeInt) 		(struct File*, int);
	void			(*writeLong) 		(struct File*, long);
	void			(*writeString)	(struct File*, TString*);
} TFile;

//Dclaration des mthodes statiques
char* File_stringToVar(TString*);

//Dclaration des constructeurs
TFile* TFile0();
TFile* TFile1(TString*);

//Dclaration des mthodes
void  		TFile_close					(TFile*);
void  		TFile_finalize			(TFile*);
//long			TFile_flush 				(TFile*);
void  		TFile_open					(TFile*);
BOOL			TFile_exists				(TFile*);
BOOL			TFile_readBoolean 	(TFile*);
char			TFile_readChar			(TFile*);
char*			TFile_readChars 		(TFile*, long);
double		TFile_readDouble		(TFile*);
float			TFile_readFloat			(TFile*);
int				TFile_readInt 			(TFile*);
long			TFile_readLong 			(TFile*);
TString* 	TFile_readString		(TFile*);
//long			TFile_size 					(TFile*);
void			TFile_writeBoolean 	(TFile*, BOOL);
void			TFile_writeChar			(TFile*, char);
void			TFile_writeChars 		(TFile*, char*);
void			TFile_writeDouble		(TFile*, double);
void			TFile_writeFloat		(TFile*, float);
void			TFile_writeInt 			(TFile*, int);
void			TFile_writeLong 		(TFile*, long);
void			TFile_writeString		(TFile*, TString*);

/**
 * Ce constructeur instancie un objet de Type File.
 @since Release 1.0
*/
TFile* TFile0 () {
	TFile *this = (TFile*)malloc(sizeof(TFile));
	
	this->class  = "TFile";
	this->super  = "TObject";
	
	this->file 	 = NULL;
	this->name   = NULL;
	this->opened = FALSE;
	this->mode   = File_WRITE;
	
	this->clone		 = TObject_clone;
	this->equals	 = TObject_equals;
	this->finalize = TFile_finalize;
	this->toString = TObject_toString;
	
	this->close 			 = TFile_close;
	this->open 				 = TFile_open;
	this->exists			 = TFile_exists;
	this->readBoolean  = TFile_readBoolean;
	this->readChar 		 = TFile_readChar;
	this->readChars 	 = TFile_readChars;
	this->readDouble 	 = TFile_readDouble;
	this->readFloat		 = TFile_readFloat;
	this->readInt 		 = TFile_readInt;
	this->readLong 		 = TFile_readLong;
	this->readString 	 = TFile_readString;
	this->writeBoolean = TFile_writeBoolean;
	this->writeChar 	 = TFile_writeChar;
	this->writeChars 	 = TFile_writeChars;
	this->writeDouble  = TFile_writeDouble;
	this->writeFloat	 = TFile_writeFloat;
	this->writeInt 		 = TFile_writeInt;
	this->writeLong 	 = TFile_writeLong;
	this->writeString  = TFile_writeString;
		
	return this;
}

/**
 * Ce constructeur instancie un objet de Type File.
 @param fileName Le nom du fichier  grer.
 @since Release 1.0
*/
TFile* TFile1 (TString* fileName) {
	TFile *this;
	
	this = TFile0();
	
	this->name = c(fileName);
	
	return this;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TFile_finalize (TFile* this) {

	if (this->opened) {
		fclose(this->file);
	}

	TRY
		if (this->name != NULL) {
			this->name->finalize(this->name);
		}
		free(this);
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode ouvre le fichier selon le mode spcifi.
 @since Release 1.0
*/
void TFile_open (TFile* this) {
	
	switch (this->mode) {
		case File_WRITE: {
			this->file = fopen((this->name)->string, "w");
			break;
		}
		case File_UPDATE: {
			this->file = fopen((this->name)->string, "w+");
			break;
		}
		case File_EXPEND: {
			this->file = fopen((this->name)->string, "a");
			break;
		}
		case File_EXPENDUPDATE: {
			this->file = fopen((this->name)->string, "a+");
			break;
		}
		case File_READ: {
			this->file = fopen((this->name)->string, "r");
			break;
		}
		case File_READUPDATE: {
			this->file = fopen((this->name)->string, "r+");
			break;
		}
	}
	
	this->opened = TRUE;
}

/**
 * Cette mthode ferme le fichier.
 @since Release 1.0
*/
void TFile_close (TFile* this) {
	
	fclose(this->file);
	
	this->opened = FALSE;
}

/**
 * Cette mthode crit un boolean dans le fichier.
 @param boolean Le boolean  crire.
 @since Release 1.0
*/
void TFile_writeBoolean (TFile* this, BOOL boolean) {
	
	fwrite(&boolean, sizeof(BOOL), 1, this->file);
}

/**
 * Cette mthode crit un caractre dans le fichier.
 @param character Le caractre  crire.
 @since Release 1.0
*/
void TFile_writeChar (TFile* this, char character) {
	
	fwrite(&character, sizeof(char), 1, this->file);
}

/**
 * Cette mthode crit un nombre  virgule de type double dans le fichier.
 @param number Le nombre  crire.
 @since Release 1.0
*/
void TFile_writeDouble (TFile* this, double number) {
	
	fwrite(&number, sizeof(double), 1, this->file);
}

/**
 * Cette mthode crit un nombre  virgule de type float dans le fichier.
 @param number Le nombre  crire.
 @since Release 1.0
*/
void TFile_writeFloat (TFile* this, float number) {
	
	fwrite(&number, sizeof(float), 1, this->file);
}

/**
 * Cette mthode crit une chane de caractres ANSI dans le fichier.
 @param chars La chane de caractres ANSI  crire.
 @since Release 1.0
*/
void TFile_writeChars (TFile* this, char* chars) {
	
	fwrite(chars, sizeof(char), strlen(chars), this->file);
}

/**
 * Cette mthode crit un entier de type integer dans le fichier.
 @param integer L'entier  crire.
 @since Release 1.0
*/
void TFile_writeInt (TFile* this, int integer) {
	
	fwrite(&integer, sizeof(int), 1, this->file);
}

/**
 * Cette mthode crit un entier de type long dans le fichier.
 @param integer L'entier  crire.
 @since Release 1.0
*/
void TFile_writeLong (TFile* this, long integer) {
	
	fwrite(&integer, sizeof(long), 1, this->file);
}

/**
 * Cette mthode crit un objet de Type String dans le fichier.
 @param paramString L'objet de Type String  crire.
 @since Release 1.0
*/
void TFile_writeString (TFile* this, TString* string) {
	
	int length = strlen(string->string);
	
	this->writeInt(this, length);

	fwrite(string->string, sizeof(char), length, this->file);
}

/**
 * Cette mthode lit un boolean dans le fichier.
 @return boolean - Le boolean lu.
 @since Release 1.0
*/
BOOL TFile_readBoolean (TFile* this) {
	
	BOOL boolean;
	
	fread(&boolean, sizeof(BOOL), 1, this->file);
	
	return boolean;
}

/**
 * Cette mthode lit un caractre dans le fichier.
 @return character - Le caractre lu.
 @since Release 1.0
*/
char TFile_readChar (TFile* this) {
	
	char character;
	
	fread(&character, sizeof(char), 1, this->file);
	
	return character;
}

/**
 * Cette mthode lit un nombre  virgule de type double dans le fichier.
 @return number - Le nombre lu.
 @since Release 1.0
*/
double TFile_readDouble (TFile* this) {
	
	double number;
	
	fread(&number, sizeof(double), 1, this->file);
	
	return number;
}

/**
 * Cette mthode lit un nombre  virgule de type float dans le fichier.
 @return number - Le nombre lu.
 @since Release 1.0
*/
float TFile_readFloat (TFile* this) {
	
	float number;
	
	fread(&number, sizeof(float), 1, this->file);
	
	return number;
}

/**
 * Cette mthode lit une chane de caractres ANSI dans le fichier.
 @param length Le nombre de caractres  lire. 
 @return string La chane de caractres ANSI lu.
 @since Release 1.0
*/
char* TFile_readChars (TFile* this, long length) {
	
	char buffer[length];
	char *chars;
	
	fread(buffer, sizeof(char), length, this->file);
	
	chars = String_copyValueOf1(buffer, length);
	
	return chars;
}

/**
 * Cette mthode lit un entier de type integer dans le fichier.
 @return integer - L'entier lu.
 @since Release 1.0
*/
int TFile_readInt (TFile* this) {
	
	int integer;
	
	fread(&integer, sizeof(int), 1, this->file);
	
	return integer;
}

/**
 * Cette mthode lit un entier de type long dans le fichier.
 @return integer - L'entier lu.
 @since Release 1.0
*/
long TFile_readLong (TFile* this) {
	
	long integer;
	
	fread(&integer, sizeof(long), 1, this->file);
	
	return integer;
}

/**
 * Cette mthode lit un objet de Type String dans le fichier.
 @return string - L'objet de Type String lu.
 @since Release 1.0
*/
TString* TFile_readString (TFile* this) {

	int length = this->readInt(this);
	char buffer[length];
	TString *string;
	
	fread(buffer, sizeof(char), length, this->file);
	
	string = TString5(buffer, length);
	
	return string;
}

/**
 * Cette mthode indique si le fichier spcifi existe.
 @return exists - Vrai si le fichier existe.
 @since Release 1.0
*/
BOOL TFile_exists (TFile* this) {

	BOOL exists = TRUE;
	
	char *var;
	
	var = File_stringToVar(this->name);
	
	if (SymFind(var).folder == HS_NULL.folder)
	{
		exists = FALSE;
	}

	free(var - (this->name->length(this->name)) - 1);

	return exists;
}

/**
 * Cette mthode convertie un nom de variable TIOS le sous format
 * chaine de caractres (TString) en format VAT string.
 @param string Le nom de la variable sous le format TString
 @return var - Le nom de la variable sous le format VAT string
 @since Release 1.0
*/
char* File_stringToVar(TString* string)
{
	int length = string->length(string);
	char *var = malloc(sizeof(char) * (length + 2));
	int i;
	
	var[0] = '\0';
	
	for (i=1; i<=length; i++) {
		var[i] = string->string[i-1];
	}
	
	var[i] = '\0';
	
	return (var + length + 1);
}
