/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TFrame - Type Frame

Hirarchie:
TObject -> TComponent -> TContainer -> TFrame
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TFrame reprsente une fentre d'environement graphique.
 @since Release 1.0
*/

//Structure de l'objet de Type Frame extension de TContainer
typedef struct Frame {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Frame*	(*clone)		(struct Frame*);
	BOOL					(*equals)		(struct Frame*, struct Object*);
	void					(*finalize)	(struct Frame*);
	TString* 			(*toString) (struct Frame*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)		 	(struct Frame*);
	void							(*setBounds)	(struct Container*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Container*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TVector* contentPane;	//Le contenu du Container
	BOOL* visibility;			//La visibilit des composants
	void							(*bringOn)		(struct Container*, TComponent*);
	void							(*center)			(struct Container*, TComponent*);
	void							(*drag)				(struct Container*, TComponent*, TCursor*);
	TString* title;				//Le titre de la fentre
	void							(*setTitle)		(struct Frame*, TString*);
	void							(*add)				(struct Container*, TComponent*);
} TFrame;

//Dclaration des constructeurs
TFrame* TFrame0();
TFrame* TFrame1(TString*);
TFrame* TFrame2(char*);

//Dclaration des mthodes
TFrame*		TFrame_clone			(TFrame*);
BOOL			TFrame_equals			(TFrame*, TObject*);
void			TFrame_finalize		(TFrame*);
TString*	TFrame_toString		(TFrame*);
void			TFrame_print 			(TFrame*);
void			TFrame_setTitle 	(TFrame*, TString* title);

/**
 * Ce constructeur instancie un objet de Type Frame.
 @since Release 1.0
*/
TFrame* TFrame0 () {
	TFrame *this = (TFrame*)malloc(sizeof(TFrame));
	
	this->class  = "TFrame";
	this->super  = "TContainer";
	
	this->height 			= 0;
	this->visible 		= FALSE;
	this->width 			= 0;
	this->x						= 0;
	this->y						= 0;
	this->img					= (BITMAP*)malloc(sizeof(BITMAP));
	this->contentPane = TVector0();
	this->visibility	= NULL;
	this->title				= s("");
	
	this->clone 	 = TFrame_clone;
	this->equals 	 = TFrame_equals;
	this->finalize = TFrame_finalize;
	this->toString = TFrame_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TFrame_print;
	this->setBounds  = TContainer_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TContainer_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	
	this->bringOn		 = TContainer_bringOn;
	this->center		 = TContainer_center;
	this->drag			 = TContainer_drag;
	this->add				 = TContainer_add;

	this->setTitle	 = TFrame_setTitle;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type Frame en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TFrame* TFrame1 (TString* caption) {
	
	TFrame *this = TFrame0();
	
	this->setTitle(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type CaptionedComponent en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TFrame* TFrame2 (char* caption) {
	
	TFrame *this;
	TString *string;
	
	string = s(caption);
	
	this = TFrame1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TFrame_print(TFrame* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	int x1 = x0 + this->width - 1;
	int y1 = y0 + this->height - 1;
	int i;
	
	for (i=y0; i<y1; i++) {
		DrawLine(x0, i, x1, i, A_REVERSE);
	}
	
	DrawLine(x0, y0, x1, y0, A_NORMAL);
	DrawLine(x0, y1, x1, y1, A_NORMAL);
	DrawLine(x0, (y0+1), x0, (y1-1), A_NORMAL);
	DrawLine(x1, (y0+1), x1, (y1-1), A_NORMAL);
	
	DrawLine((x0+1), (y0+12), (x1-1), (y0+12), A_NORMAL);
	DrawStr((x0+2), (y0+2), this->title->string, A_NORMAL);
}

/**
 * Cette mthode change le titre de la fentre.
 @param title Le titre de la fentre
 @since Release 1.0
*/
void TFrame_setTitle(TFrame* this, TString* title) {

	BOOL visible = this->visible;
	
	if (visible) {
		TContainer_setVisible(this, FALSE);
	}
	
	this->title->finalize(this->title);	
	this->title = c(title);
	
	if (visible) {
		TContainer_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode retourne une chane de caractres reprsentant l'objet.
 * Pour la classe TFrame, il s'agit du titre de la fentre.
 @return string - La chane de caractre reprsentant l'objet.
 @since Release 1.0
*/
TString* TFrame_toString	(TFrame* this) {
	
	TString* string;
	
	string = c(this->title);
	
	return string;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TFrame_finalize (TFrame* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->contentPane->finalize(this->contentPane);
		if (this->visibility != NULL) {
			free(this->visibility);
		}
		this->title->finalize(this->title);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode retourne un objet de Type Frame qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TFrame* TFrame_clone(TFrame* this) {

	TFrame *clone;
	int i;
	
	clone = TFrame0();
	
	clone->height 		 = this->height;
	clone->visible		 = FALSE;
	clone->width			 = this->width;
	clone->x					 = this->x;
	clone->y					 = this->y;
	clone->contentPane = this->contentPane->clone(this->contentPane);
	clone->title			 = c(this->title);
		
	if (this->visibility != NULL)	{
		clone->visibility = malloc(this->contentPane->size * sizeof(BOOL));
		for (i = 0; i < this->contentPane->size; i++) {
			clone->visibility[i] = this->visibility[i];
		}
	}
	
	return clone;
}

/**
 * Cette mthode compare l'objet  un autre objet pour tablir une galit.
 * Dans le cas d'un objet de Type Frame, une comparaison est tablie
 * entre les reprsentations en chane de caractres des objets.
 @param object0 L'objet  comparer.
 @return result - Vrai si les deux objets sont gaux.
 @since Release 1.0
*/
BOOL TFrame_equals (TFrame* this, TObject* object0) {
	
	BOOL result;
	
	if (this->title->equals(this->title, object0->toString(object0))) {
		result = TRUE;
	}
	else {
		result = FALSE;
	}
	
	return result;
}
