/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TLabel - Type Label

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TLabel
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TLabel reprsente une tiquette texte.
 @since Release 1.0
*/

#define Label_COLOR_WHITE 0
#define Label_COLOR_BLACK 1

//Structure de l'objet de Type Label extension de TCaptionedComponent
typedef struct Label {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct Label*		(*clone)		(struct Label*);
	BOOL						(*equals)		(struct CaptionedComponent*, struct Object*);
	void						(*finalize)	(struct Label*);
	TString* 				(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct Label*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
	BOOL opaque;			//L'opacit de l'tiquette
	int color;				//La couleur du texte de l'tiquette
	void							(*setColor)		(struct Label*, int);
	void							(*setOpaque)	(struct Label*, BOOL);
} TLabel;

//Dclaration des constructeurs
TLabel* TLabel0();
TLabel* TLabel1(TString*);
TLabel* TLabel2(char*);

//Dclaration des mthodes
TLabel*							TLabel_clone				(TLabel*);
void								TLabel_finalize			(TLabel*);
void								TLabel_print				(TLabel*);
void								TLabel_setColor			(TLabel*, int);
void								TLabel_setOpaque		(TLabel*, BOOL);

/**
 * Ce constructeur instancie un objet de Type Label.
 @since Release 1.0
*/
TLabel* TLabel0 () {
	TLabel *this = (TLabel*)malloc(sizeof(TLabel));
	
	this->class  = "TLabel";
	this->super  = "TCaptionedComponent";
	
	this->height 	= 0;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	this->opaque	= FALSE;
	this->color		= Label_COLOR_BLACK;
	
	this->clone 	 = TLabel_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TLabel_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TLabel_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;
	this->setColor	 = TLabel_setColor;
	this->setOpaque	 = TLabel_setOpaque;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type Label en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TLabel* TLabel1 (TString* caption) {
	
	TLabel *this = TLabel0();
	
	this->setCaption(this, caption);
		
	return this;
}

/**
 * Ce constructeur instancie un objet de Type Label en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TLabel* TLabel2 (char* caption) {
	
	TLabel *this;
	TString *string;
	
	string = s(caption);
	
	this = TLabel1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type Label qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TLabel* TLabel_clone(TLabel* this) {

	TLabel *clone;
	
	clone = TLabel0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->caption	= c(this->caption);
	clone->opaque		= this->opaque;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TLabel_finalize (TLabel* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode affiche le composant  l'cran.
 @since Release 1.0
*/
void TLabel_print(TLabel* this) {
	
	int x0 = this->x;
	int y0 = this->y;
	
	int attr;
	
	if (this->opaque) {
		attr = A_REPLACE;
	}
	else {
		attr = A_NORMAL;
	}
	
	DrawStr((x0+2), (y0+2), this->caption->string, attr);
	
	if (this->color == Label_COLOR_WHITE)
	{
		DrawStr((x0+2), (y0+2), this->caption->string, A_XOR);
	}
}

/**
 * Cette mthode change la couleur du texte de l'tiquette.
 @param color La couleur du texte
 @since Release 1.0
*/
void TLabel_setColor(TLabel* this, int color) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->color = color;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}

/**
 * Cette mthode change l'opacit de l'tiquette.
 @param opaque Spcifie si l'tiquette est opaque
 @since Release 1.0
*/
void TLabel_setOpaque(TLabel* this, BOOL opaque) {

	BOOL visible = this->visible;
	
	if (visible) {
		TComponent_setVisible(this, FALSE);
	}
	
	this->opaque = opaque;
	
	if (visible) {
		TComponent_setVisible(this, TRUE);
	}
}
