/*
QUSOFT MICROSYSTMES
C Object Oriented Language

Classe: TRadioButton - Type RadioButton

Hirarchie:
TObject -> TComponent -> TCaptionedComponent -> TButton -> TCheckButton -> TRadioButton
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * La classe TRadioButton reprsente un bouton que l'utilisateur peut cocher.
 * Les RadioButtons font partie d'un rseau o seul un RadioButton peut tre coch
 *  la fois.
 @since Release 1.0
*/

//Structure de l'objet de Type RadioButton extension de TButton
typedef struct RadioButton {
	char *class; 	//Le nom de la classe
	char *super; 	//Le nom de la superclasse
	struct RadioButton*	(*clone)		(struct RadioButton*);
	BOOL								(*equals)		(struct CaptionedComponent*, struct Object*);
	void								(*finalize)	(struct RadioButton*);
	TString* 						(*toString) (struct CaptionedComponent*);
	int height;  	//La hauteur du composant
	BITMAP *img;	//L'image de la partie de l'cran cache par le composant
	BOOL visible; //La visibilit du composant
	int width;		//La largeur du composant
	int x;				//La position horizontale du composant
	int y;				//La position verticale du composant
	void							(*move)				(struct Component*, int, int);
	void							(*print)			(struct CheckButton*);
	void							(*setBounds)	(struct Component*, int, int, int, int);
	void							(*setHeight)	(struct Component*, int);
	void							(*setVisible)	(struct Component*, BOOL);
	void							(*setWidth)		(struct Component*, int);
	void							(*setX)				(struct Component*, int);
	void							(*setY)				(struct Component*, int);
	TString* caption;	//L'tiquette du composant
	void							(*setCaption)	(struct CaptionedComponent*, TString*);
	BOOL checked;
	void							(*setChecked)	(struct RadioButton*, BOOL);
	TComponentNetwork* network; //Le rseau auquel le RadioButton fait partie
} TRadioButton;

//Dclaration des constructeurs
TRadioButton* TRadioButton0();
TRadioButton* TRadioButton1(TString*);
TRadioButton* TRadioButton2(char*);
TRadioButton* TRadioButton3(TString*, TComponentNetwork*);
TRadioButton* TRadioButton4(char*, TComponentNetwork*);

//Dclaration des mthodes
TRadioButton*					TRadioButton_clone			(TRadioButton*);
void									TRadioButton_finalize		(TRadioButton*);
void									TRadioButton_setChecked	(TRadioButton*, BOOL);

/**
 * Ce constructeur instancie un objet de Type RadioButton.
 @since Release 1.0
*/
TRadioButton* TRadioButton0 () {
	TRadioButton *this = (TRadioButton*)malloc(sizeof(TRadioButton));
	
	this->class  = "TRadioButton";
	this->super  = "TCheckButton";
	
	this->height 	= 15;
	this->visible = FALSE;
	this->width 	= 0;
	this->x				= 0;
	this->y				= 0;
	this->img			= (BITMAP*)malloc(sizeof(BITMAP));
	this->caption	= s("");
	this->checked = FALSE;
	this->network = NULL;
	
	this->clone 	 = TRadioButton_clone;
	this->equals 	 = TCaptionedComponent_equals;
	this->finalize = TRadioButton_finalize;
	this->toString = TCaptionedComponent_toString;
	
	this->move 			 = TComponent_move;
	this->print 		 = TCheckButton_print;
	this->setBounds  = TComponent_setBounds;
	this->setHeight  = TComponent_setHeight;
	this->setVisible = TComponent_setVisible;
	this->setWidth 	 = TComponent_setWidth;
	this->setX 			 = TComponent_setX;
	this->setY 			 = TComponent_setY;
	this->setCaption = TCaptionedComponent_setCaption;

	this->setChecked = TRadioButton_setChecked;

	return this;
}

/**
 * Ce constructeur instancie un objet de Type RadioButton en
 * spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TRadioButton* TRadioButton1 (TString* caption) {
	
	TRadioButton *this = TRadioButton0();
	
	this->setCaption(this, caption);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type RadioButton en
 * spcifiant le texte de l'tiquette  l'aide d'une chane de
 * caractres ANSI.
 @param caption Le texte de l'tiquette
 @since Release 1.0
*/
TRadioButton* TRadioButton2 (char* caption) {
	
	TRadioButton *this;
	TString *string;
	
	string = s(caption);
	
	this = TRadioButton1(string);
	
	string->finalize(string);
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type RadioButton en lui assignant
 * un rseau de RadioButtons et en spcifiant le texte de l'tiquette.
 @param caption Le texte de l'tiquette
 @param network Le rseau de RadioButtons
 @since Release 1.0
*/
TRadioButton* TRadioButton3 (TString* caption, TComponentNetwork* network) {
	
	TRadioButton *this = TRadioButton1(caption);
	
	this->network = network;
	
	return this;
}

/**
 * Ce constructeur instancie un objet de Type RadioButton en lui assignant
 * un rseau de RadioButtons et en spcifiant le texte de l'tiquette  l'aide
 * d'une chane de caractres ANSI.
 @param caption Le texte de l'tiquette
 @param network Le rseau de RadioButtons
 @since Release 1.0
*/
TRadioButton* TRadioButton4 (char* caption, TComponentNetwork* network) {
	
	TRadioButton *this = TRadioButton2(caption);
	
	this->network = network;
	
	return this;
}

/**
 * Cette mthode retourne un objet de Type RadioButton qui est une copie de l'objet.
 @return clone - Une copie de l'objet.
 @since Release 1.0
*/
TRadioButton* TRadioButton_clone(TRadioButton* this) {

	TRadioButton *clone;
	
	clone = TRadioButton0();
	
	clone->height 	= this->height;
	clone->visible	= FALSE;
	clone->width		= this->width;
	clone->x				= this->x;
	clone->y				= this->y;
	clone->checked	= this->checked;
	clone->network	= this->network;
	
	return clone;
}

/**
 * Cette mthode libre l'espace mmoire occupe par l'objet, dtruisant ce dernier.
 @since Release 1.0
*/
void TRadioButton_finalize (TRadioButton* this) {
	
	this->setVisible(this, FALSE);
	
	TRY {
		this->caption->finalize(this->caption);
		free(this->img);
		free(this);
	}
	ONERR {}
	ENDTRY
}

/**
 * Cette mthode permet de cocher le bouton.
 @param checked Vrai si le composant doit tre coch
 @since Release 1.0
*/
void TRadioButton_setChecked (TRadioButton* this, BOOL checked) {
	
	if (checked) {
		if (this->network->active != NULL) {
			((TRadioButton*)this->network->active)->setChecked(this->network->active, FALSE);
		}
		this->network->active = this;
	}
	
	TCheckButton_setChecked(this, checked);
}
