/**
 * A simple graphic environment application.  Displays a window with a text box.
 * To quit, the user must write "quit" in the text box.
*/

/*
	Copyrigth 2001-2002 Frdric Brown
*/

/*
    This file is part of COOL.

    COOL is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    COOL is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with COOL; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define USE_TI89              // Produce .89z File
#define USE_TI92PLUS          // Produce .9xz File

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "COOL_LANG.h"
#include "COOL_UTILS.h"
#include "COOL_IO.h"
#include "COOL_X.h"

// Main Function
void _main(void)
{
	TFrame *f = TFrame0();//The main window
	TTextField *tf = TTextField0();//The text field where the user will type his password
	TCursor *c = TCursor0();//The cursor used by the user to interact with GUI
	TLabel *lbl0 = TLabel1(s("This calc has been locked."));//A label
	TLabel *lbl1 = TLabel1(s("Password:"));//A label
	TLabel *lbl2 = TLabel0();//A label to print error messages
	TCheckButton *btn = TCheckButton1(s("Exit"));//The exit button
	TVector *v;//A vector, in fact this vector will point to the vector contained in the window
	BOOL exit = FALSE;//A flag used to know if the application must quit
	BOOL locked = TRUE;//A flag used to know if the calculator is locked
	
	tf->setMasked(tf, TRUE); //The text field will mask the user input
	
	v = f->contentPane;//Assign the content pane of the window (faster to type v instead of f->contentPane!)
	
	lbl0->move(lbl0, 5, 15);//Moves the lbl0 to the desired position
	v->addElement(v, lbl0);//Adds the lbl0 to the content pane
		
	lbl1->move(lbl1, 5, 30);//Moves the lbl1 to the desired position
	v->addElement(v, lbl1);//Adds the lbl1 to the content pane
	
	lbl2->move(lbl2, 5, 60);//Moves the lbl1 to the desired position
	v->addElement(v, lbl2);//Adds the lbl1 to the content pane
	
	btn->setChecked(btn, TRUE);//The quit button will be checked (for a grayed effect)
	btn->move(btn, 5, 90);//Moves the btn to the desired position
	v->addElement(v, btn);//Adds the btn to the content pane
	
	tf->setBounds(tf, 80, 30, 120, 45);//Sets the bounds of tf
	v->addElement(v, tf);//Adds the tf to the content pane
	
	f->setTitle(f, s("Screen locked"));//Sets the title of the window
	f->setBounds(f, 0, 0, LCD_WIDTH - 1, LCD_HEIGHT - 1);//Sets the bounds of the window
	
	f->setVisible(f, TRUE);//Shows the window
	
	//Shows the component in the window
	lbl0->setVisible(lbl0, TRUE);
	lbl1->setVisible(lbl1, TRUE);
	lbl2->setVisible(lbl2, TRUE);
	btn->setVisible(btn, TRUE);
	tf->setVisible(tf, TRUE);
	
	while (!exit) {
		c->use(c);//Give the user control of the cursor
		/* ... When the user presses ENTER ... */
		//If the calculator is locked and the cursor over the textfield
		if (locked && c->isOver(c, tf)) {
			tf->use(tf);//Give the user control of the textfield (to type a password)
			//if the password is ok (if the caption of the textfield is equal to "quit")
			if (tf->caption->equals(tf->caption, s("quit"))) {
				tf->setCaption(tf, s(""));//Erase the caption of the textfield
				lbl2->setCaption(lbl2, s("Password accepted."));//Print "Password accepted"
				locked = FALSE;//Indicate that the calculator is not longer locked
				btn->setChecked(btn, FALSE);//uncheck the quit button
			}
			else {
				lbl2->setCaption(lbl2, tf->caption->concat(tf->caption, s(" is not a valid password.")));//If the password is not equal to "quit" then print an error message
			}
		}
		//Else if the calculator is no longuer locked and the cursor is over the quit button
		else if (!locked && c->isOver(c, btn)) {
			exit = TRUE;//Exits the while next loop
		}
	}
	
}
